/**************************************************************************/
/*                                                                        */
/*  Thibaut Balabonski, Sylvain Conchon, Jean-Christophe Filliâtre,       */
/*  Kim Nguyen, Laurent Sartre                                            */
/*                                                                        */
/*  Informatique - MP2I/MPI - CPGE 1re et 2e années.                      */
/*  Cours et exercices corrigés. Éditions Ellipses, 2022.                 */
/*                                                                        */
/*  https://www.informatique-mpi.fr/                                      */
/*                                                                        */
/**************************************************************************/

// file réalisée avec une liste simplement chaînée

#include "prelude.h"

#include "list.h"
#include "queue.h"

struct Queue { int size; list *first, *last; };
  /* invariant size = 0 et first == last == NULL
            ou size > 0 et first, last != NULL */

queue *queue_create(void) {
  queue *q = malloc(sizeof(struct Queue));
  q->size = 0;
  q->first = q->last = NULL;
  return q;
}

int queue_size(queue *q) {
  return q->size;
}

bool queue_is_empty(queue *q) {
  return queue_size(q) == 0;
}

void queue_enqueue(queue *q, int x) {
  if (queue_is_empty(q)) {
    q->first = q->last = list_cons(x, NULL);
    q->size = 1;
    return;
  }
  list *c = list_cons(x, NULL);
  q->last->next = c;
  q->last = c;
  q->size++;
}

int queue_peek(queue *q) {
  assert(!queue_is_empty(q));
  return q->first->value;
}

int queue_dequeue(queue *q) {
  assert(!queue_is_empty(q));
  int v = q->first->value;
  list *p = q->first;
  q->first = p->next;
  free(p);
  if (q->first == NULL) q->last = NULL;
  q->size--;
  return v;
}

void queue_delete(queue *q) {
  list_delete(q->first);
  free(q);
}
